/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.field.model;

import com.vividsolutions.jts.geom.Geometry;
import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.render.GeoTranslator;
import nl.cloudfarming.client.geoviewer.render.GeometricalWidget;
import org.jdesktop.swingx.color.ColorUtil;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.model.ObjectState;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages({"ShapeWidget tooltip name=Name:","ShapeWidget tooltip value=Value:"})
public class ShapeWidget extends GeometricalWidget {

    private final Shape shape;
    
    public ShapeWidget(ObjectScene scene, GeoTranslator geoTranslator, Geometrical geometrical, Palette palette, Geometry envelope) {
        super(scene, geoTranslator, geometrical, palette, envelope);
        assert geometrical instanceof Shape:"Provided geometrical should be castable to Shape";
        shape = (Shape)geometrical;
        setForeground(palette.getColorForValue(shape.getImportType()));
        updateTooltip();
        shape.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if(Shape.PROP_IMPORT_TYPE.equals(evt.getPropertyName())){
                    setForeground(getPalette().getColorForValue(shape.getImportType()));
                    repaint();
                }
                if(Shape.PROP_NAME.equals(evt.getPropertyName())|| Shape.PROP_VALUE.equals(evt.getPropertyName())){
                    updateTooltip();
                }
            }
        });
    }
    
    private void updateTooltip(){
        StringBuilder sb = new StringBuilder(Bundle.ShapeWidget_tooltip_name());
        sb.append(shape.getName());
        sb.append("\n");
        sb.append(Bundle.ShapeWidget_tooltip_value());
        sb.append(shape.getValue());
        setToolTipText(sb.toString());
        
    }
    
    /**
     * overide to prevent state change to change color, we want value based colors in the ShapeWidget
     * instead of state based. Use alpha channel of statebase colors to set alpha on current foreground
     * @param previousState
     * @param state 
     */
    @Override
    protected void notifyStateChanged(ObjectState previousState, ObjectState state) {
        Color c = getPalette().getColorForState(state);
        Color foreground = getForeground();
        setForeground(ColorUtil.setAlpha(foreground, c.getAlpha()));
        repaint();
    }
    
}
